@@ -239,20 +239,23 @@ public:
 
     T value(int frameNo) const
     {
-        if (frames_.front().start_ >= frameNo)
-            return frames_.front().value_.start_;
-        if (frames_.back().end_ <= frameNo) return frames_.back().value_.end_;
+        if (!frames_.empty()) {
+            if (frames_.front().start_ >= frameNo)
+                return frames_.front().value_.start_;
+            if (frames_.back().end_ <= frameNo) return frames_.back().value_.end_;
 
-        for (const auto &keyFrame : frames_) {
-            if (frameNo >= keyFrame.start_ && frameNo < keyFrame.end_)
-                return keyFrame.value(frameNo);
+            for (const auto &keyFrame : frames_) {
+                if (frameNo >= keyFrame.start_ && frameNo < keyFrame.end_)
+                    return keyFrame.value(frameNo);
+            }
         }
         return {};
     }
 
     float angle(int frameNo) const
     {
-        if ((frames_.front().start_ >= frameNo) ||
+        if (frames_.empty() ||
+            (frames_.front().start_ >= frameNo) ||
             (frames_.back().end_ <= frameNo))
             return 0;
 
@@ -265,6 +268,8 @@ public:
 
     bool changed(int prevFrame, int curFrame) const
     {
+        if (frames_.empty()) return false;
+
         auto first = frames_.front().start_;
         auto last = frames_.back().end_;
 
