@@ -64,6 +64,17 @@ RAPIDJSON_DIAG_PUSH
 RAPIDJSON_DIAG_OFF(effc++)
 #endif
 
+#ifdef _WIN32
+#include <windows.h>
+#include <shlwapi.h>
+
+#include <string_view>
+#endif
+
+#ifndef PATH_MAX
+#define PATH_MAX MAX_PATH
+#endif
+
 using namespace rapidjson;
 
 using namespace rlottie::internal;
@@ -797,6 +808,102 @@ static std::string convertFromBase64(const std::string &str)
     return b64decode(b64Data, length);
 }
 
+namespace
+{
+   #ifdef _WIN32
+   std::wstring ToStdWString(const std::string &str)
+   {
+      std::wstring wstr;
+      int          nchars = ::MultiByteToWideChar(CP_UTF8, 0, str.data(), (int)str.length(), 0, 0);
+      if ( nchars > 0 )
+      {
+        wstr.resize( nchars );
+        ::MultiByteToWideChar( CP_UTF8, 0, str.data(), (int)str.length(),
+                               const_cast<wchar_t *>( wstr.c_str() ),
+                                nchars );
+      }
+
+      return wstr;
+   }
+
+   std::string ToStdString(const std::wstring &wstr)
+   {
+       std::string str;
+       int         nchars = ::WideCharToMultiByte( CP_UTF8, 0, wstr.data(), (int)wstr.length(), NULL, NULL, NULL, NULL );
+       if ( nchars > 0 )
+       {
+           str.resize(nchars);
+           ::WideCharToMultiByte( CP_UTF8, 0, wstr.data(), (int)wstr.length(),
+                                  const_cast<char *>(str.c_str()), nchars, NULL, NULL );
+       }
+
+       return str;
+   }
+   #endif
+
+   bool Canonicalize(const char *path, char *resolved_path)
+   {
+#ifdef _WIN32
+       std::wstring wpath = ToStdWString( path );
+       std::wstring wresolved_path;
+       wresolved_path.resize( PATH_MAX );
+       if ( PathCanonicalizeW(const_cast<wchar_t*>(wresolved_path.data()), wpath.c_str() ) )
+       {
+           std::string path = ToStdString(wresolved_path);
+           strcpy_s( resolved_path, path.length() * sizeof( char ), path.c_str() );
+
+           return true;
+       }
+
+       return false;
+#else
+       return realpath(path, resolved_path);
+#endif
+   }
+}
+
+static bool isResourcePathSafe(const std::string& baseDir, const std::string& userPath)
+{
+    char resolvedBase[PATH_MAX] = {};
+    char resolvedTarget[PATH_MAX] = {};
+
+    // Resolve base directory
+    if (!Canonicalize(baseDir.c_str(), resolvedBase))
+    {
+
+#ifdef DEBUG_PARSER
+        vWarning << "Error: Cannot resolve base path: " << baseDir.c_str();
+#endif
+        return false;
+    }
+
+    // Resolve target path
+    std::string fullPath = baseDir;
+    if (!baseDir.empty() && baseDir.back() != '/') fullPath += "/";
+    fullPath += userPath;
+
+    if (!Canonicalize(fullPath.c_str(), resolvedTarget)) {
+#ifdef DEBUG_PARSER
+        vWarning << "Error: Cannot resolve target path: " << fullPath.c_str();
+#endif
+        return false;
+    }
+
+    std::string base(resolvedBase);
+    std::string target(resolvedTarget);
+
+    // Ensure target starts with base
+    bool result = target.compare(0, base.length(), base) == 0 &&
+         (target.length() == base.length() || target[base.length()] == '/');
+
+    if (!result) {
+#ifdef DEBUG_PARSER
+        vWarning << "Error: Dangerous path blocked: " << fullPath.c_str();
+#endif
+    }
+    return result;
+}
+
 /*
  *  std::to_string() function is missing in VS2017
  *  so this is workaround for windows build
@@ -819,7 +926,7 @@ model::Asset *LottieParserImpl::parseAsset()
     auto        asset = allocator().make<model::Asset>();
     std::string filename;
     std::string relativePath;
-    bool        embededResource = false;
+    bool        embeddedResource = false;
     EnterObject();
     while (const char *key = NextObjectKey()) {
         if (0 == strcmp(key, "w")) {
@@ -832,7 +939,7 @@ model::Asset *LottieParserImpl::parseAsset()
         } else if (0 == strcmp(key, "u")) { /* relative image path */
             relativePath = GetStringObject();
         } else if (0 == strcmp(key, "e")) { /* relative image path */
-            embededResource = GetInt();
+            embeddedResource = GetInt();
         } else if (0 == strcmp(key, "id")) { /* reference id*/
             if (PeekType() == kStringType) {
                 asset->mRefId = GetStringObject();
@@ -859,14 +966,18 @@ model::Asset *LottieParserImpl::parseAsset()
         }
     }
 
-    if (asset->mAssetType == model::Asset::Type::Image) {
-        if (embededResource) {
-            // embeder resource should start with "data:"
-            if (filename.compare(0, 5, "data:") == 0) {
+    if (asset->mAssetType == model::Asset::Type::Image && !filename.empty()) {
+        if (embeddedResource) {
+            // embedded resource should start with "data:"
+            // URL Scheme: "data:[<mediatype>][;base64],<data>"
+            if (filename.compare(0, 5, "data:") == 0 && filename.find(',') != std::string::npos) {
                 asset->loadImageData(convertFromBase64(filename));
             }
         } else {
-            asset->loadImagePath(mDirPath + relativePath + filename);
+            // reject dangerous paths
+            if (isResourcePathSafe(mDirPath, relativePath + filename)) {
+                asset->loadImagePath(mDirPath + relativePath + filename);
+            }
         }
     }
 
@@ -2368,17 +2479,16 @@ public:
 
 static char* uncompressZip(const char * str, size_t length)
 {
-    const char* errMsg = "Failed to unzip dotLottie!";
-
     auto zip = zip_stream_open(str, length, 0, 'r');
     if (!zip) {
-        vCritical << errMsg;
+        vCritical << "Failed to unzip dotLottie: read fail!";
         return nullptr;
     }
 
-    //Read a representive animation
+    // Read a representative animation
     if (zip_entry_openbyindex(zip, 1)) {
-        vCritical << errMsg;
+        vCritical << "Failed to unzip dotLottie: open by index fail!";
+        zip_stream_close(zip);
         return nullptr;
     }
 
@@ -2389,7 +2499,20 @@ static char* uncompressZip(const char * str, size_t length)
     zip_entry_close(zip);
     zip_stream_close(zip);
 
-    return buf;
+    if (buf == nullptr || bufSize == 0) {
+        vCritical << "Failed to unzip dotLottie: buffer is empty!";
+        return nullptr;
+    }
+
+    char* terminatedBuf = static_cast<char*>(realloc(buf, bufSize + 1));
+    if (!terminatedBuf) {
+        free(buf);
+        vCritical << "Failed to unzip dotLottie: failed to allocate!";
+        return nullptr;
+    }
+
+    terminatedBuf[bufSize] = '\0';
+    return terminatedBuf;
 }
 
 static bool checkDotLottie(const char * str)
@@ -2399,6 +2522,32 @@ static bool checkDotLottie(const char * str)
     else return false;
 }
 
+std::shared_ptr<model::Composition> parseImpl(char* input,
+                                              std::string dir_path,
+                                              model::ColorFilter filter)
+{
+    LottieParserImpl obj(input, std::move(dir_path), std::move(filter));
+
+    if (!obj.VerifyType()) {
+        vWarning << "Input data is not Lottie format!";
+        return {};
+    }
+
+    obj.parseComposition();
+    auto composition = obj.composition();
+    if (composition) {
+        composition->processRepeaterObjects();
+        composition->updateStats();
+
+#ifdef LOTTIE_DUMP_TREE_SUPPORT
+        ObjectInspector inspector;
+        inspector.visit(composition.get(), "");
+#endif
+    }
+
+    return composition;
+}
+
 std::shared_ptr<model::Composition> model::parse(char *             str,
                                                  size_t             length,
                                                  std::string        dir_path,
@@ -2409,30 +2558,16 @@ std::shared_ptr<model::Composition> model::parse(char *             str,
     auto dotLottie = checkDotLottie(str);
     if (dotLottie) {
         input = uncompressZip(str, length);
-    }
-
-    LottieParserImpl obj(input, std::move(dir_path), std::move(filter));
-
-    if (dotLottie) free(input);
-
-    if (obj.VerifyType()) {
-        obj.parseComposition();
-        auto composition = obj.composition();
-        if (composition) {
-            composition->processRepeaterObjects();
-            composition->updateStats();
-
-#ifdef LOTTIE_DUMP_TREE_SUPPORT
-            ObjectInspector inspector;
-            inspector.visit(composition.get(), "");
-#endif
-
-            return composition;
+        if (!input) {
+            vWarning << "Failed to decompress .lottie archive.";
+            return {};
         }
     }
 
-    vWarning << "Input data is not Lottie format!";
-    return {};
+    auto result = parseImpl(input, std::move(dir_path), std::move(filter));
+
+    if (dotLottie) free(input);
+    return result;
 }
 
 RAPIDJSON_DIAG_POP
